@@ -1,23 +1,26 @@
 """
 Copyright 2020 Marco Dal Molin et al.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
+This file is part of SuperflexPy.
 
-    http://www.apache.org/licenses/LICENSE-2.0
+SuperflexPy is free software: you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
+SuperflexPy is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with SuperflexPy. If not, see <https://www.gnu.org/licenses/>.
 
 This file is part of the SuperflexPy modelling framework. For details about it,
 visit the page https://superflexpy.readthedocs.io
 
 CODED BY: Marco Dal Molin
-DESIGNED BY: Marco Dal Molin, Fabrizio Fenicia
+DESIGNED BY: Marco Dal Molin, Fabrizio Fenicia, Dmitri Kavetski
 
 This file contains the implementation of some elements needed to replicate the
 model Hymod (Boyle et al., 2001; Wagner et al., 2001).
@@ -122,6 +125,7 @@ class UpperZone(ODEsElement):
         fluxes = self._num_app.get_fluxes(fluxes=self._fluxes_python,
                                           S=self.state_array,
                                           S0=self._solver_states,
+                                          dt=self._dt,
                                           **self.input,
                                           **{k[len(self._prefix_parameters):]: self._parameters[k] for k in self._parameters},
                                           )
@@ -148,6 +152,7 @@ class UpperZone(ODEsElement):
         fluxes = self._num_app.get_fluxes(fluxes=self._fluxes_python,
                                           S=S,
                                           S0=self._solver_states,
+                                          dt=self._dt,
                                           **self.input,
                                           **{k[len(self._prefix_parameters):]: self._parameters[k] for k in self._parameters},
                                           )
@@ -157,7 +162,7 @@ class UpperZone(ODEsElement):
     # PROTECTED METHODS
 
     @staticmethod
-    def _fluxes_function_python(S, S0, ind, P, Smax, m, beta, PET):
+    def _fluxes_function_python(S, S0, ind, P, Smax, m, beta, PET, dt):
         # TODO: handle time variable parameters (Smax) -> overflow
 
         if ind is None:
@@ -168,7 +173,7 @@ class UpperZone(ODEsElement):
                     - P * (1 - (1 - (S / Smax))**beta),
                 ],
                 0.0,
-                S0 + P
+                S0 + P * dt
             )
         else:
             return (
@@ -178,13 +183,13 @@ class UpperZone(ODEsElement):
                     - P[ind] * (1 - (1 - (S / Smax[ind]))**beta[ind]),
                 ],
                 0.0,
-                S0 + P[ind]
+                S0 + P[ind] * dt[ind]
             )
 
     @staticmethod
-    @nb.jit('Tuple((UniTuple(f8, 3), f8, f8))(optional(f8), f8, i4, f8[:], f8[:], f8[:], f8[:], f8[:])',
+    @nb.jit('Tuple((UniTuple(f8, 3), f8, f8))(optional(f8), f8, i4, f8[:], f8[:], f8[:], f8[:], f8[:], f8[:])',
             nopython=True)
-    def _fluxes_function_numba(S, S0, ind, P, Smax, m, beta, PET):
+    def _fluxes_function_numba(S, S0, ind, P, Smax, m, beta, PET, dt):
         # TODO: handle time variable parameters (Smax) -> overflow
 
         return (
@@ -194,7 +199,7 @@ class UpperZone(ODEsElement):
                 - P[ind] * (1 - (1 - (S / Smax[ind]))**beta[ind]),
             ),
             0.0,
-            S0 + P[ind]
+            S0 + P[ind] * dt[ind]
         )
 
 
@@ -274,6 +279,7 @@ class LinearReservoir(ODEsElement):
         fluxes = self._num_app.get_fluxes(fluxes=self._fluxes_python,  # I can use the python method since it is fast
                                           S=self.state_array,
                                           S0=self._solver_states,
+                                          dt=self._dt,
                                           **self.input,
                                           **{k[len(self._prefix_parameters):]: self._parameters[k] for k in self._parameters},
                                           )
@@ -283,7 +289,7 @@ class LinearReservoir(ODEsElement):
     # PROTECTED METHODS
 
     @staticmethod
-    def _fluxes_function_python(S, S0, ind, P, k):
+    def _fluxes_function_python(S, S0, ind, P, k, dt):
 
         if ind is None:
             return (
@@ -292,7 +298,7 @@ class LinearReservoir(ODEsElement):
                     - k * S,
                 ],
                 0.0,
-                S0 + P
+                S0 + P * dt
             )
         else:
             return (
@@ -301,13 +307,13 @@ class LinearReservoir(ODEsElement):
                     - k[ind] * S,
                 ],
                 0.0,
-                S0 + P[ind]
+                S0 + P[ind] * dt[ind]
             )
 
     @staticmethod
-    @nb.jit('Tuple((UniTuple(f8, 2), f8, f8))(optional(f8), f8, i4, f8[:], f8[:])',
+    @nb.jit('Tuple((UniTuple(f8, 2), f8, f8))(optional(f8), f8, i4, f8[:], f8[:], f8[:])',
             nopython=True)
-    def _fluxes_function_numba(S, S0, ind, P, k):
+    def _fluxes_function_numba(S, S0, ind, P, k, dt):
         # This method is used only when solving the equation
 
         return (
@@ -316,5 +322,5 @@ class LinearReservoir(ODEsElement):
                 - k[ind] * S,
             ),
             0.0,
-            S0 + P[ind]
+            S0 + P[ind] * dt[ind]
         )
